﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Http;

using Vintasoft.Data;
using Vintasoft.Imaging.Annotation.Formatters;
using Vintasoft.Imaging.Annotation.Web.Api2Controllers;
using Vintasoft.Imaging.Annotation.Web.Services;
using Vintasoft.Imaging.Web.Api2Controllers;
using Vintasoft.Imaging.Web.Services;

namespace AspNetMvcAnnotationDemo.Controllers
{
    /// <summary>
    /// A Web API controller that handles HTTP requests from clients and
    /// allows to manipulate annotations on server.
    /// </summary>
    public class MyVintasoftAnnotationCollectionApiController : VintasoftAnnotationCollectionApi2Controller
    {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MyVintasoftAnnotationCollectionApiController"/> class.
        /// </summary>
        public MyVintasoftAnnotationCollectionApiController() :
            base()
        {
            // define custom serialization binder for correct deserialization of triangle and mark annotations
            AnnotationSerializationBinder.Current = new CustomAnnotationSerializationBinder();

            // set the custom annotation formatter
            Formatter = new CustomAnnotationJsonFormatter();
            DataStoragePath = @"D:\1\";
        }

        #endregion



        #region Properties

        /// <summary>
        /// Gets or sets base path for data storages.
        /// </summary>
        public string DataStoragePath
        {
            get; set;
        }

        #endregion



        #region Methods

        /// <summary>
        /// Creates the <see cref="VintasoftAnnotationCollectionWebService"/>
        /// that handles HTTP requests from clients and allows to annotate images.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>
        /// The <see cref="VintasoftAnnotationCollectionWebService"/>
        /// that handles HTTP requests from clients and allows to annotate images.
        /// </returns>
        /// <remarks>
        /// You need override the <see cref="CreateImageTilesCacheManager"/> method
        /// if you need change any parameter of image tiles cache (directory name,
        /// cache life time, etc).<br />
        /// <br />
        /// You need override the <see cref="CreateThumbnailsCacheManager"/> method
        /// if you need change any parameter of thumbnails cache (directory name,
        /// cache life time, etc).<br />
        /// <br />
        /// You need override the <see cref="CreateAnnotationsCacheManager"/> method
        /// if you need change any parameter of annotations cache (directory name,
        /// cache life time, etc).
        /// </remarks>
        /// <seealso cref="CreateImageTilesCacheManager"/>
        /// <seealso cref="CreateThumbnailsCacheManager"/>
        /// <seealso cref="CreateAnnotationsCacheManager"/>
        protected override VintasoftAnnotationCollectionWebService CreateWebService(string sessionId)
        {
            IDataStorage sessionDataStorage = CreateSessionDataStorage(sessionId);
            IDataStorage resourcesDataStorage = CreateGlobalResourcesDataStorage();
            IDataStorage serializedDocumentsDataStorage = CreateSerializedDocumentsDataStorage(sessionId);
            return new VintasoftAnnotationCollectionWebService(
                Formatter,
                sessionDataStorage,
                resourcesDataStorage,
                serializedDocumentsDataStorage,
                CreateImageTilesCacheManager(sessionId),
                CreateThumbnailsCacheManager(sessionId),
                CreateAnnotationsCacheManager(sessionId));
        }

        /// <summary>
        /// Creates a data storage for current session.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/wwwroot/UploadedImageFiles[/{SessionID}]" directory.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override IDataStorage CreateSessionDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "UploadedImageFiles";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), sessionId);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a data storage for global resources.
        /// </summary>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/Resources" directory.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        protected override IDataStorage CreateGlobalResourcesDataStorage()
        {
            // data sub directory
            string subDirectory = "Resources";
            // full path to data storage
            string path = Path.Combine(DataStoragePath, subDirectory);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a data storage for serialized documents.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/VintasoftCache/{SessionId}/SerializedDocuments" folder.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        protected override IDataStorage CreateSerializedDocumentsDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string serializedDocumentsDirectory = "SerializedDocuments";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, serializedDocumentsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a cache manager that manages cache of image tiles.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>A cache manager that manages cache of image tiles.</returns>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override ImageTilesCacheManager CreateImageTilesCacheManager(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string tilesDirectory = "Tiles";
            // full path to the working directory of cache manager
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, tilesDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            ImageTilesCacheManager imageTilesCacheManager = new ImageTilesCacheManager(path);

            // cache life time in minutes
            int cacheLifeTime = 60;
            imageTilesCacheManager.CacheLifeTime = cacheLifeTime;

            return imageTilesCacheManager;
        }

        /// <summary>
        /// Creates a cache manager that manages cache of thumbnails.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>A cache manager that manages cache of thumbnails.</returns>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override ThumbnailsCacheManager CreateThumbnailsCacheManager(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string thumbnailsDirectory = "Thumbnails";
            // full path to the working directory of cache manager
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, thumbnailsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            ThumbnailsCacheManager thumbnailsCacheManager = new ThumbnailsCacheManager(path);

            // cache life time in minutes
            int cacheLifeTime = 60;
            thumbnailsCacheManager.CacheLifeTime = cacheLifeTime;

            return thumbnailsCacheManager;
        }

        /// <summary>
        /// Creates cache manager that manages cache of annotation collections.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>A cache manager that manages cache of annotation collections.</returns>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override AnnotationsCacheManager CreateAnnotationsCacheManager(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string annotationsDirectory = "Annotations";
            // full path to the working directory of cache manager
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, annotationsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            AnnotationsCacheManager annotationsCacheManager = new AnnotationsCacheManager(path);

            // cache life time in minutes
            int cacheLifeTime = 60;
            annotationsCacheManager.CacheLifeTime = cacheLifeTime;

            return annotationsCacheManager;
        }

        #endregion

    }
}