﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Http;

using Vintasoft.Data;
using Vintasoft.Imaging.Web.Api2Controllers;
using Vintasoft.Imaging.Web.Services;

namespace AspNetMvcAnnotationDemo.Controllers
{
    /// <summary>
    /// A Web API controller that handles HTTP requests from clients and
    /// allows to manipulate files on server.
    /// </summary>
    public class MyVintasoftFileApiController : VintasoftFileApi2Controller
    {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MyVintasoftFileApiController"/> class.
        /// </summary>
        public MyVintasoftFileApiController()
            : base()
        {
            DataStoragePath = @"D:\1\";
        }

        #endregion



        #region Properties

        /// <summary>
        /// Gets or sets base path for data storages.
        /// </summary>
        public string DataStoragePath
        {
            get; set;
        }

        #endregion



        #region Methods

        /// <summary>
        /// Returns a list of files uploaded during current HTTP session.
        /// </summary>
        /// <param name="session">Identifier of HTTP session.</param>
        /// <returns>Dictionary that provides a mapping from filename to URL of file.</returns>
        [HttpPost]
        public UploadedFilesListResponseParams GetUploadedFilesUrl([FromBody] string session)
        {
            UploadedFilesListResponseParams answer = new UploadedFilesListResponseParams();
            IDataStorage storage = CreateSessionDataStorage(session);
            if (storage != null)
            {
                List<string> files = new List<string>();
                string[] allFiles = storage.GetKeys();
                for (int i = 0; i < allFiles.Length; i++)
                {
                    files.Add(allFiles[i]);
                }
                answer.files = files.ToArray();
            }
            answer.success = true;
            return answer;
        }
        
        /// <summary>
         /// Creates the <see cref="VintasoftFileWebService"/>
         /// that handles HTTP requests from clients and allows to manipulate files on a server.
         /// </summary>
         /// <param name="sessionId">The session identifier.</param>
         /// <returns>The <see cref="VintasoftFileWebService"/>
         /// that handles HTTP requests from clients and allows to manipulate files on a server.</returns>
        protected override VintasoftFileWebService CreateWebService(string sessionId)
        {
            IDataStorage storage = CreateSessionDataStorage(sessionId);
            IDataStorage serializedDocumentsDataStorage = CreateSerializedDocumentsDataStorage(sessionId);
            return new VintasoftFileWebService(storage, serializedDocumentsDataStorage);
        }

        /// <summary>
        /// Creates a data storage for current session.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/wwwroot/UploadedImageFiles[/{SessionID}]" directory.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override IDataStorage CreateSessionDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "UploadedImageFiles";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), sessionId);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a data storage for serialized documents.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/VintasoftCache/{SessionId}/SerializedDocuments" folder.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        protected override IDataStorage CreateSerializedDocumentsDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string serializedDocumentsDirectory = "SerializedDocuments";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, serializedDocumentsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a cache manager that manages cache of image tiles.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>A cache manager that manages cache of image tiles.</returns>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override ImageTilesCacheManager CreateImageTilesCacheManager(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string tilesDirectory = "Tiles";
            // full path to the working directory of cache manager
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, tilesDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            ImageTilesCacheManager imageTilesCacheManager = new ImageTilesCacheManager(path);

            // cache life time in minutes
            int cacheLifeTime = 60;
            imageTilesCacheManager.CacheLifeTime = cacheLifeTime;

            return imageTilesCacheManager;
        }

        /// <summary>
        /// Creates a cache manager that manages cache of thumbnails.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>A cache manager that manages cache of thumbnails.</returns>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override ThumbnailsCacheManager CreateThumbnailsCacheManager(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string thumbnailsDirectory = "Thumbnails";
            // full path to the working directory of cache manager
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, thumbnailsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            ThumbnailsCacheManager thumbnailsCacheManager = new ThumbnailsCacheManager(path);

            // cache life time in minutes
            int cacheLifeTime = 60;
            thumbnailsCacheManager.CacheLifeTime = cacheLifeTime;

            return thumbnailsCacheManager;
        }

        #endregion

    }
}