﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Http;

using Vintasoft.Data;
using Vintasoft.Imaging.Web.Api2Controllers;
using Vintasoft.Imaging.Web.Services;

namespace AspNetMvcAnnotationDemo.Controllers
{
    /// <summary>
    /// A Web API controller that handles HTTP requests from clients and
    /// allows to manage an image collection.
    /// </summary>
    public class MyVintasoftImageCollectionApiController : VintasoftImageCollectionApi2Controller
    {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MyVintasoftImageCollectionApiController"/> class.
        /// </summary>
        public MyVintasoftImageCollectionApiController()
            : base()
        {
            DataStoragePath = @"D:\1\";
        }

        #endregion



        #region Properties

        /// <summary>
        /// Gets or sets base path for data storages.
        /// </summary>
        public string DataStoragePath
        {
            get; set;
        }

        #endregion



        #region Methods

        /// <summary>
        /// Creates the <see cref="VintasoftImageCollectionWebService"/>
        /// that handles HTTP requests from clients and allows to manage an image collection.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The <see cref="VintasoftImageCollectionWebService"/>
        /// that handles HTTP requests from clients and allows to manage an image collection.</returns>
        /// <remarks>
        /// You need override the <see cref="CreateSessionDataStorage"/> method
        /// if you need change any parameter of data storage.
        /// </remarks>
        /// <seealso cref="CreateSessionDataStorage"/>
        protected override VintasoftImageCollectionWebService CreateWebService(string sessionId)
        {
            IDataStorage sessionDataStorage = CreateSessionDataStorage(sessionId);
            IDataStorage serializedDocumentsDataStorage = CreateSerializedDocumentsDataStorage(sessionId);
            return new VintasoftImageCollectionWebService(sessionDataStorage, serializedDocumentsDataStorage);
        }

        /// <summary>
        /// Creates a data storage for current session.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/wwwroot/UploadedImageFiles[/{SessionID}]" directory.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        /// <exception cref="ArgumentNullException">Thrown if
        /// <see cref="IsEmptySessionSupported"/> = <b>true</b> and <i>sessionId</i> is <b>null</b>.
        /// </exception>
        /// <exception cref="NotSupportedException">Thrown if
        /// the physical path to the root of the application is not defined,
        /// i.e. service is used in not ASP.NET application.
        /// </exception>
        protected override IDataStorage CreateSessionDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "UploadedImageFiles";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), sessionId);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        /// <summary>
        /// Creates a data storage for serialized documents.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        /// <remarks>By default method creates the <see cref="StreamDataStorage"/>
        /// storage that works with "{Project Path}/VintasoftCache/{SessionId}/SerializedDocuments" folder.<br/>
        /// <br/>
        /// <b>Important:</b> The overridden method must return data storage that
        /// can store <see cref="Stream"/> objects.
        /// </remarks>
        protected override IDataStorage CreateSerializedDocumentsDataStorage(string sessionId)
        {
            if (sessionId == null)
            {
                if (IsEmptySessionSupported)
                    sessionId = "";
                else
                    throw new ArgumentNullException("sessionId");
            }

            // data sub directory
            string subDirectory = "VintasoftCache";
            string serializedDocumentsDirectory = "SerializedDocuments";
            // full path to data storage
            string path = Path.Combine(Path.Combine(DataStoragePath, subDirectory), Path.Combine(sessionId, serializedDocumentsDirectory));

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
            return new StreamDataStorage(path);
        }

        #endregion

    }
}